Module: FatDate

Defined in:
lib/fat_date.rb,
lib/fat_date/date.rb,
lib/fat_date/version.rb,
lib/fat_date/date_time.rb

Overview

Gem Overview (extracted from README.org by gem_docs)

  • Introduction ~fat_date~ collects core extensions for the Date class to make it more useful in financial applications, including:
  • determining when a =Date= is a federal or NYSE holiday with Presidential decrees included,
  • determining when a =Date= is part of a larger calendar-related "chunk," such as a year, half, quarter, bimonth, month, semimonth, or week,
  • calculating Easter for a =Date's= year, a date on which some "movable feasts" depend, and
  • parsing so-called "specs" that allow the beginning or ending =Date= of a larger period of time to be returned, a facility put to good use in the [[https://github.com/ddoherty03/fat_period][FatPeriod]] gem.

Defined Under Namespace

Modules: Date, DateTime

Constant Summary collapse

VERSION =
"0.2.0"